# Minimal Decomposition Field of Polynomial over Ratinal
#
#   by Shin-ichiro Hara
#
# Version 1.00 (2001.07.22)

require "polynomial"
require "polynomial-factor"
require "m-polynomial"
require "groebner-basis"
require "residue-class-ring"

def symmetric_product(objs)
  a = [1]
  objs.each do |x|
    ax = a.collect{|t| t * x}
    a.each_index do |i|
      if i > 0 && i < a.size
	a[i] = a[i] + ax[i-1]
      end
    end
    a.push ax.last
  end
  a
end

def MinimalDecompositionField(f, facts = f.factorize)
  vars = []
  var = "a"
  facts.each do |f, n|
    dim = f.deg
    next if dim < 2
    dim.downto 1 do
      vars.unshift var
#      vars.push var
      var = var.succ
    end
  end

  pr = MPolynomial(f.ground)
  vs = pr.vars(*vars)

  i = 0
  fs = []
  facts.each do |f, n|
    dim = f.deg
    if dim > 1
      sym = symmetric_product(vs[i, dim])
      fa = (1..dim).collect{|j| sym[j] - f[dim-j]*(-1)**j}
      fs.concat fa
      i += dim
    end
  end

  pr.sysvar(:basis, Groebner.basis(fs))
  pr.class_eval "
    def inverse
      qs, bs = Groebner.basis_coeff([self] + @@basis)
      unless bs == [1]
	raise self.to_s + \" can't be inverted.\"
      end
      qs[0][0]
    end
  "
  rs = ResidueClassRing(pr, pr.basis)
  rs.sysvar(:vars, pr.vars.collect{|v| rs[v]})
  rs
end

if __FILE__ == $0
  require "rational"
  class Rational < Numeric
    def inspect; to_s; end
  end
  
  P = Polynomial(Rational, "t")
  t = P.var

  if false
    f = t**2 - 2
    K  = MinimalDecompositionField(f)
    a, b = K.vars
    p a
    p b
    p 1/(b+2)
#    p 1/(b-1)
  else
    f = t**3 - 2
    K  = MinimalDecompositionField(f)
    a, b, c = K.vars
    p a + b + c
    p a*b + b*c + c*a
    p a*b*c
    p ((a-2).inverse)
    p ((a-1)*(b-1)*(c-1))
  end
end
