# Gussian Elimination Algorism
#
#   by Shin-ichiro Hara
#
# Version 1.01 (2001.07.22)

module GaussianElimination
#ElementaryOpeartion
  def swap_r!(i, j)
    a, b = row(i), row(j)
    set_row(i, b).set_row(j, a)
  end

  def swap_r(i, j)
    dup.swap_r!(i, j)
  end

  def swap_c!(i, j)
    a, b = column(i), column(j)
    set_column(i, b).set_column(j, a)
  end

  def swap_c(i, j)
    dup.swap_c!(i, j)
  end

  def multiply_r!(i, c)
    set_row(i, row(i).collect{|x| x * c})
  end

  def multiply_r(i, c)
    dup.multiply_r!(i, c)
  end

  def multiply_c!(j, c)
    set_column(j, column(j).collect{|x| x * c})
  end

  def multiply_c(j, c)
    dup.multiply_c!(j, c)
  end

  def mix_r!(i, j, c)
    row_i = row(i)
    row_j = row(j)
    set_row(i, (0...csize).collect{|k| row_i[k] + row_j[k]*c})
  end

  def mix_r(i, j, c)
    dup.mix_r!(i, j, c)
  end

  def mix_c!(i, j, c)
    column_i = column(i)
    column_j = column(j)
    set_column(i, (0...rsize).collect{|k| column_i[k] + column_j[k]*c})
  end

  def mix_c(i, j, c)
    dup.mix_c!(i, j, c)
  end

#

  def left_eliminate!(pivots = [])
    mtype = SquareMatrix(ground, rsize)
    inv = mtype.unity
    k = ground.unity
    pi = 0
    each_j do |j|
      pi.upto(rsize-1) do |i|
	unless self[i, j].zero?
	  pivots.push j

	  swap_r!(pi, i) if i != pi
	  inv.swap_r!(pi, i) if i != pi
	  k = -k

#	  p 77777
#	  p [ground.unity, self[pi, j], ground.unity.type, self[pi, j].type]
#	  p ground.unity / self[pi, j]
#	  p 88888
	  c = ground.unity / self[pi, j]
	  multiply_r!(pi, c)
	  inv.multiply_r!(pi, c)
	  k = k * c

	  each_i do |i0|
	    next if i0 == pi
	    d = - self[i0, j]
	    mix_r!(i0, pi, d)
	    inv.mix_r!(i0, pi, d)
	  end
	  pi += 1
	  break
	end
      end
    end
    [inv, k]
  end

  def left_inverse
    dup.left_eliminate![0]
  end

  def left_sweep
    m = dup
    m.left_eliminate!
    m
  end

  def step_matrix?
    pivots = []
    i = j = 0
    each_j do |j|
      if !self[i, j].zero?
	0.upto rsize - 1 do |i0|
	  return nil if i0 != i && !self[i0, j].zero?
	end
	pivots << j
	return pivots if (i += 1) >= rsize
      else
	(i+1).upto rsize - 1 do |i0|
	  return nil if i0 != i && !self[i0, j].zero?
	end
      end
    end
    pivots
  end

  def kernel_basis
    if pivots =  step_matrix?
      m = self
    else
      m = left_sweep
      pivots = m.step_matrix?
    end
    basis = []
    v = Vector(ground, csize)
    (0...csize).each do |i|
      unless pivots.include? i
	base = v.vector{|j|
	  if j == i
	    ground.unity
	  elsif k = pivots.index(j)
	    -m[k, i]
	  else
	    ground.zero
	  end
	}
	basis.push base
      end
    end
    basis
  end

  def determinant_by_elimination #ground ring must be a field
    m = dup
    inv, k = m.left_eliminate!
    s = ground.unity
    (0...size).each do |i|
      s *= m[i, i]
    end
    s / k
  end
end

class MatrixAlgebra
  include GaussianElimination
  class SquareMatrix < MatrixAlgebra
    def inverse; left_inverse; end
  end
end

if $0 == __FILE__
  require "m-polynomial"
  require "polynomial"
  require "rational"
  require "matrix-algebra"
  class Rational < Numeric
    def inspect; to_s; end
  end


  G = MatrixAlgebra(Rational, 2, 2)
  a = G[[0, 0], [1, -1]]
  p a
  p a.kernel_basis

#  R = MPolynomial(Integer)
  R = MPolynomial(Rational)
  a, b, c, d, e, f, g, h, i, j, k, l, x, y, z = R.vars("abcdefghijklxyz")
  M3 = MatrixAlgebra(R, 3, 4)
  m = M3[
    [a, b, c, d],
    [e, f, g, h],
    [i, j, k, l]
  ]
  m.display
  m.swap_r(0, 2).display
#  m.set_column(1, [1,2,3]).display
#  puts
  m.swap_c(0, 2).display
  m.multiply_r(2, x).display
  m.multiply_c(0, x).display
  m.mix_r(0, 2, x).display
  m.mix_c(0, 2, x).display

  u, z = R.unity, R.zero
  require "residue-class-ring"
  F = ResidueClassRing(Integer, 2)
#  M = SquareMatrix(Rational, 8)
  M = SquareMatrix(F, 8)
  m = M[
    "10001101".scan(/./).collect{|x|x.to_i},
    "00000011".scan(/./).collect{|x|x.to_i},
    "01001011".scan(/./).collect{|x|x.to_i},
    "00001111".scan(/./).collect{|x|x.to_i},
    "00101011".scan(/./).collect{|x|x.to_i},
    "00001001".scan(/./).collect{|x|x.to_i},
    "00011011".scan(/./).collect{|x|x.to_i},
    "00000111".scan(/./).collect{|x|x.to_i}
  ]
  m = m - m.unity
  puts "m = "
  m.display
  m.kernel_basis.each do |b|
    puts b.type
    puts b
    #[1, 0, 0, 0, 0, 0, 0, 0]
    #[0, 1, 1, 1, 1, 1, 1, 0]
    #[0, 1, 1, 0, 1, 0, 0, 1]
  end
end
