# String <=> Polynomial <=> MPolynomial translator
#
#   by Shin-ichiro Hara
#
# Version 2.0 (2001.04.08)

class Polynomial
  def self.convert_to(other)
    if other <= MPolynomial
      g = self
      vs = []
      while g <= Polynomial
	vs.push g.var
	g = g.ground
      end
      MPolynomial.create(g, *vs)
    else
      raise "unkown type (#{other})"
    end
  end

  def value_on(ring)
    e = ring.zero
    x = ring.var(var)
    reverse_each do |c|
      e = e * x + (ground <= Polynomial ? c.value_on(ring) : c)
    end
    e
  end

  def var_swap
    k = Polynomial(ground.ground, type.variable)
    ring = Polynomial(k, ground.variable)
    e = ring.zero
    x = ring.var
    y = ring.ground.var
    reverse_each do |c|
      e = e * y + c.evaluate(x)
    end
    e
  end
end

class MPolynomial
  def self.convert_to(other)
    if other <= Polynomial
      Polynomial.create(ground, *(vars.reverse))
    else
      raise "unkown type (#{other})"
    end
  end

  def value_on(ring)
    e = ring.zero
    each do |idx, c|
      e = e + idx.value_on(ring, c)
    end
    e
  end
end

class MIndex < Array
  def value_on(ring, c)
    vars = ring.vars
    e = ring.unity
    each_with_index do |n, i|
      e = e * vars[i] ** n
    end
    e * c
  end
end


if __FILE__ == $0
  require "m-polynomial"
  require "polynomial"
#  MPolynomial.extend Polynomial2Polynomial
#  Polynomial.extend Polynomial2Polynomial

  P = Polynomial.create(Integer, "x", "y", "z")
  x, y, z = P.vars
  f = x**2 + y**2 + z**2 - x*y - y*z - z*x
  MP = P.convert_to(MPolynomial)
  x, y, z = MP.vars
  g = x**2 + y**2 + z**2 - x*y - y*z - z*x
  p f = f.value_on(MP) #=> z^2 - zy - zx + y^2 - yx + x^2
  p f == g             #=> true

  P0 = MP.convert_to(Polynomial)
  x, y, z = P0.vars
  g = x**2 + y**2 + z**2 - x*y - y*z - z*x
  p f = f.value_on(P0)
  p f == g

  Px = Polynomial(Integer, "x")
  x = Px.var
  Pxy = Polynomial(Px, "y")
  y0 = Pxy.var

  Py = Polynomial(Integer, "y")
  y = Py.var
  Pyx = Polynomial(Py, "x")
  x0 = Pyx.var

  f = y0**2 * x - y0**3 * x**2
  g = y**2 * x0 - y**3 * x0**2
  p f
  p g
  p f.var_swap
end
