# Expression of Divisors
#
#   by Shin-ichiro Hara
#
# Version 1.0 (2001.04.18)

module PolynomialFactorization
  class Factors
    include Enumerable
    def to_a; @bone; end
    def initialize(array = []); @bone = array; end
    def each(&b); @bone.each(&b); end
    def size; @bone.size; end
    def fact(i); @bone[i].first; end
    def empty?; @bone.empty?; end
    def head; @bone.first; end
    def tail; type.new(@bone[1..-1]); end
    def push(other); @bone.push other; self; end
    def unshift(other); @bone.unshift(other); self; end
    def collect(&b); type.new(@bone.collect(&b)); end
    def map(&b); type.new(@bone.collect{|f, i| [yield(f), i]}); end
    def collect!(&b); @bone.collect!(&b); self; end
    def map!(&b); @bone.collect!{|f, i| [yield(f), i]}; self; end
    def multiply(other)
      flag = false
      @bone.each do |fact|
	f, i = fact
	if f == other
	  fact[1] = fact[1]+ 1
	  flag = true
	  break
	end
      end
      @bone.push [other, 1] unless flag
      self
    end
    
    def *(o); type.new(dup.multiply(o).to_a); end
    alias << multiply

    def pi
      x = 1
      @bone.each do |f_i|
	f, i = f_i
	if i
	  x *= f**i
	else
	  x *= f
	end
      end
      x
    end

    def to_s
      a, b = @bone.size == 1 && @bone.first.last == 1? ["", ""] : ["(", ")"]
      @bone.collect{|x|
	"#{a}#{x.first}#{b}" + (x.last > 1 ? "^#{x.last}" : "")
      }.join("")
    end
    alias inspect to_s

    def each_factor0 #create non trivial powers
      if size > 0
	fact0, e0 = head
	yield fact0
	tail.each_factor do |fact, e|
	  yield fact
	end
	tail.each_factor do |fact, e|
	  yield fact0*fact # e is ignored
	end
      end
    end

    def each_factor #create non trivial powers
      idx0 = (0...size).to_a
      indices = []
      idx0.each do |i|
	break if i >= size
	indices.push [idx0[i...size], [i], fact(i)]
      end
      
      avoid = []
      until indices.empty?
	idx, rdx, f = indices.shift
	i = idx.first
	next if avoid.find{|x| !(rdx & x).empty? }
#	print rdx.inspect + " : "
	r = yield(f)
	if r
	  avoid.push rdx
	  next
	end
	idx.each_index do |k|
	  if k > 0 and k < idx.size
	    idx1 = idx[k..-1]
	    rdx1 = rdx + [idx[k]]
	    next if avoid.find{|x| !(rdx1 & x).empty?}
	    f1 = f * fact(idx[k])
	    indices.push [idx1, rdx1, f1]
	  end
	end
      end
    end


    def fact_all(f)
      facts = type.new
      each do |g, e|
	i = 0
	loop do
	  q, r = f.divmod_ED g
	  if r.zero?
	    f = q
	    i += 1
	    if f.deg <= 0
	      facts.push [g, i]
	      break
	    end
	  else
	    facts.push [g, i]
	    break
	  end
	end
      end
      facts
    end

    def correct!(f)
      k = f.ground.unity
      each do |g, i|
	k *= g.lc ** i
      end
      m = f.lc / k
      unshift([m*f.unity, 1]) unless m == f.ground.unity
      self
    end
  end
end

if $0 == __FILE__
  facts = PolynomialFactorization::Factors.new
  facts << 2 << 3 << 5 << 7 << 11 << 13
#  p facts
#  p facts.pi
  k = 0
  facts.each_factor2 do |x|
    k += 1
    p x
    x == 21
  end
  p k
end
